@@ -146,7 +146,11 @@ RPB_FUNCTION_VOID_2(METHOD(serialize_to_file), Rcpp::XPtr<GPB::Message> message,
 RPB_FUNCTION_1(Rcpp::RawVector, METHOD(get_payload), Rcpp::XPtr<GPB::Message> message) {
 
     /* create a raw vector of the appropriate size */
+#if GOOGLE_PROTOBUF_VERSION < 3006001
+    R_xlen_t size = message->ByteSize();
+#else
     R_xlen_t size = message->ByteSizeLong();
+#endif
     Rcpp::RawVector payload(size);
 
     /* fill the array */
@@ -250,7 +254,11 @@ RPB_FUNCTION_1(S4_Descriptor, METHOD(descriptor), Rcpp::XPtr<GPB::Message> messa
 }
 
 RPB_XP_METHOD_0(METHOD(as_character), GPB::Message, DebugString)
-RPB_XP_METHOD_0(METHOD(bytesize), GPB::Message, ByteSizeLong)
+#if GOOGLE_PROTOBUF_VERSION < 3006001
+  RPB_XP_METHOD_0(METHOD(bytesize), GPB::Message, ByteSize)
+#else
+  RPB_XP_METHOD_0(METHOD(bytesize), GPB::Message, ByteSizeLong)
+#endif
 
 RPB_FUNCTION_2(int, METHOD(field_size), Rcpp::XPtr<GPB::Message> message, SEXP field) {
     const GPB::FieldDescriptor* field_desc = getFieldDescriptor(message, field);
@@ -431,16 +439,26 @@ RPB_FUNCTION_1(Rcpp::CharacterVector, METHOD(fieldNames), Rcpp::XPtr<GPB::Messag
  * Returns the JSON representation of the message.
  *
  * @param xp external pointer to a Message
+ * @param preserve_proto_field_names If FALSE (the default) convert field names to camelCase.
+ * @param always_print_primitive_fields Whether to return the default value for missing primitive fields. (default false)
  *
  * @return JSON representation of the message as a single element R character vector (STRSXP)
  */
-RPB_FUNCTION_1(Rcpp::CharacterVector, METHOD(as_json), Rcpp::XPtr<GPB::Message> message) {
+RPB_FUNCTION_3(Rcpp::CharacterVector, METHOD(as_json), Rcpp::XPtr<GPB::Message> message,
+               bool preserve_proto_field_names,
+               bool always_print_primitive_fields) {
 #ifdef PROTOBUF_JSON_UTIL
     GPB::util::JsonPrintOptions opts;
     opts.add_whitespace = true;
+    opts.preserve_proto_field_names = preserve_proto_field_names;
+    opts.always_print_primitive_fields = always_print_primitive_fields;
 
     std::string buf;
-    GPB::util::Status status = GPB::util::MessageToJsonString(*message, &buf, opts);
+    #if GOOGLE_PROTOBUF_VERSION < 4022000
+        GPB::util::Status status = GPB::util::MessageToJsonString(*message, &buf, opts);
+    #else
+        absl::Status status = GPB::util::MessageToJsonString(*message, &buf, opts);
+    #endif
     if (!status.ok()) {
         Rcpp::stop(status.ToString().c_str());
     }
@@ -1163,6 +1181,17 @@ RPB_FUNCTION_VOID_4(METHOD(set_field_values), Rcpp::XPtr<GPB::Message> message,
     }
 }
 
-#undef METHOD
+//#undef METHOD
+
+/**
+ * Return the TextFormat of message
+ * @param xp external pointer to the Message
+ */
+RPB_FUNCTION_1(std::string, METHOD(print_text_format),
+               Rcpp::XPtr<GPB::Message> message) {
+  std::string message_text;
+  GPB::TextFormat::PrintToString(*message, &message_text);
+  return message_text;
+}
 
 }
